package org.amse.asves.skinCreator;

import org.amse.asves.skinCreator.reader.*;
import org.amse.asves.skinCreator.view.*;
import org.amse.asves.skinCreator.writer.*;
import org.amse.asves.skinCreator.wsz.*;
import org.amse.asves.skinCreator.exceptions.*;
import org.amse.asves.skinCreator.model.*;
import org.amse.asves.skinCreator.model.impl.*;
import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;

/**
 *  Main
 *     
 * @author 
 *
 */
public final class Main {
	
	/**
	 *       
	 * @param frame      
	 * @param message 
	 * @param exit     
	 */
	private static void showErrorDialog(JFrame frame, String message, boolean exit){
		if (message == null){
			message = "   ";
			exit = true;
		}
		JOptionPane.showMessageDialog(frame, message, 
				"Error", JOptionPane.ERROR_MESSAGE);
		if (exit) System.exit(0);
	}
	
	/**
	 *    About
	 * @param frame      
	 */
	private static void showAbout(JFrame frame){
		JOptionPane.showMessageDialog(frame, Constants.ABOUT_MESSAGE, 
				" Skin Creator v." + Constants.VERSION, JOptionPane.INFORMATION_MESSAGE);
	}
		/**
	*  
	 * @param extracter       
	 *    
	 * @return 
	 * @throws CriticalError      
	 * @throws ErrorReadingProject      
	 * @throws NullPointerException  extracter  Null
	 */
	public static IProject initializeProject(SkinFilesManager extracter)
	throws CriticalError, ErrorReadingProject, NullPointerException{
		
		IProject project = new Project();
		ProjectReader reader = new ProjectReader();
		InputStream arrow = Main.class.getResourceAsStream("view/arrow.gif");
		InputStream vis = Main.class.getResourceAsStream("view/milkdrop.gif");
		
		reader.initializePictures(vis, arrow);
		reader.read(project, extracter);
		return project;
	}
	
	/**
	 *          
	 *     @param frame
	 * @param extracter       
	 *    
	 * @return 
	 */
	private static IProject initializeAndShowErrors(JFrame frame, SkinFilesManager extracter){
		try{
		try{
		try{
			try{
				try{
					extracter.renew();
					return initializeProject(extracter);
				}catch(NullPointerException e){
					showErrorDialog(frame, e.getMessage(), false);
					return null;
				}
			}catch(CriticalError e){
				showErrorDialog(frame, e.getMessage(), true);
				return null;
			}
		}catch(ErrorReadingProject e){
			showErrorDialog(frame, e.getMessage(), false);
			return null;
		}
		}catch(Exception e){
			showErrorDialog(frame, e.getMessage(), false);
			return null;
		}
	}catch(Exception e){
		showErrorDialog(frame, e.getMessage(), false);
		return null;
	}
	}
	
	/**
	 *          
	 *         
	 * @param frame - JFrame     
	 * @param extractor -   
	 * @param projectWriter -   
	 * @param viewer - 
	 * @param mode -       
	 */
	private static void manageSave(final JFrame frame, 
			final SkinFilesManager extractor, 
			final SkinWriter projectWriter, 
			final Viewer viewer, final int mode){
		try{
		int x;
		if (mode != 3){
			x = JOptionPane.showConfirmDialog(frame, "    \n " +
					"    ?");
		}else{
			x = JOptionPane.showConfirmDialog(frame, "    \n " +
					"      ?");
		}
		if (x == JOptionPane.NO_OPTION) {
			Viewer.save();
			if (mode == 1) 
				open(frame, extractor, projectWriter, viewer);
			if (mode == 2) 
				setNew(frame, extractor, projectWriter, viewer);
			if (mode == 3)
				System.exit(0);
			
		} else { if (x == JOptionPane.YES_OPTION) {
			
			Viewer.save();
			
			save(extractor, projectWriter, frame);
			
			if (mode == 1) 
				open(frame, extractor, projectWriter, viewer);
			if (mode == 2) 
				setNew(frame, extractor, projectWriter, viewer);
			if (mode == 3)
				System.exit(0);
			
		}else { if (x == JOptionPane.CANCEL_OPTION) {
			
		}
		}
		}
		}catch(Exception e){
			showErrorDialog(frame, e.getMessage(), false);
		}
	}
	
	/**
	 *         
	 * @param frame - -
	 *        
	 * @param extractor
	 * @param projectWriter
	 * @param viewer
	 */
	private static void open(JFrame frame, SkinFilesManager extractor, 
			SkinWriter projectWriter, Viewer viewer){
		try{
		String path;
		if (!extractor.currentIsDefault()){
			path = extractor.getCurrentSkinFileName();
		}else {
			path = "";//"org/amse/asves/skinCreator";
		}
		JFileChooser chooser = new JFileChooser(path);
		chooser.setFileFilter(new SkinFileFilter());
		chooser.showOpenDialog(frame);
		File selectedFile = chooser.getSelectedFile();
		if (selectedFile != null){
			try{
		
				extractor.setCurrentSkin(selectedFile);
				IProject project = initializeProject(extractor);
				projectWriter.refresh(project.getSkin(), extractor);
				Viewer s_viewer = new Viewer(project);
				
				frame.getContentPane().removeAll();
				frame.getContentPane().add(s_viewer);
				frame.paintComponents(frame.getGraphics());
			}catch(ErrorReadingProject e){
				showErrorDialog(frame, e.getMessage(), false);
			}
		}
		}catch(Exception e){
			e.printStackTrace();
			showErrorDialog(frame, e.getMessage(), false);
		}
	}
	
	/**
	 *   
	 */
	private static void setNew(JFrame frame, SkinFilesManager extractor,
			SkinWriter projectWriter, Viewer viewer){
		try{
		extractor.renew();
		IProject project = initializeAndShowErrors(frame, extractor);
		frame.getContentPane().removeAll();
		Viewer s_viewer = new Viewer(project);
		frame.getContentPane().add(s_viewer);
		projectWriter.refresh(project.getSkin(), extractor);
		frame.paintComponents(frame.getGraphics());
		}catch(Exception e){
			showErrorDialog(frame, e.getMessage(), false);
		}
	}
	
	/**
	 *    ...
	 * @param extractor
	 * @param projectWriter
	 * @param frame
	 */
	private static void saveAs(SkinFilesManager extractor,
			SkinWriter projectWriter, JFrame frame){
		
		JFileChooser chooser = new JFileChooser("org/amse/asves/skinCreator");
		SkinFileFilter filter = new SkinFileFilter();
		chooser.setFileFilter(filter);
		chooser.showSaveDialog(frame);
		File selectedFile = chooser.getSelectedFile();
		
		if (selectedFile != null){
			String filePath = selectedFile.getPath();
			if (!filter.accept(selectedFile))
				filePath += ".wsz";
			try{
				projectWriter.writeAll(filePath);
			}catch(WSZWriteException e){
				showErrorDialog(frame, e.getMessage(), false);
			}	
			try{
				extractor.setCurrentSkin(new File(filePath));
			} catch(ErrorReadingProject e){
				showErrorDialog(frame, e.getMessage(), false);
			}
			Viewer.save();
		}
	}
	
	/**
	 *       ,  
	 *     " "
	 * @param extractor
	 * @param projectWriter
	 * @param frame
	 */
	private static void save(SkinFilesManager extractor,
			SkinWriter projectWriter, JFrame frame){
		if (!extractor.currentIsDefault()){
			try{
				projectWriter.writeAll();
			}catch(WSZWriteException e){
				showErrorDialog(frame, e.getMessage(), false);
			}
			Viewer.save();
		}else{
			saveAs(extractor, projectWriter, frame);
		}
	}
	
	/**
	 *     @param file   -
	 *    @param is
	 * @throws ErrorReadingProject    
	 */
	public static void createTempFileFromStream(InputStream is, File file)
	throws ErrorReadingProject{
		
		FileOutputStream fos;
		try{
			fos = new FileOutputStream(file);
		}catch(FileNotFoundException e){
			throw new ErrorReadingProject(e.getMessage());
		}
		if (is != null){
			try {
				SkinFilesManager.copyStream (is, fos);
			} catch(CannotCopyStreamException e){
				throw new CriticalError("    ");
			}
    	
   	 		try{
   	 			is.close();
   	 		}catch(IOException e){
   	 			throw new ErrorReadingProject("    ");
   	 		} 
   	 	}
	}
	
	/**
	 *  
	 * @param args
	 */
	public static void main(String[] args) {
		
		final JFrame frame = new JFrame("Skin Creator Version " + Constants.VERSION);
		Options.initializeOptions(Options.RIGHT_SIDED);
		frame.setSize(Options.FRAME_WIDTH, Options.FRAME_HEIGHT);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		/**
		 *  2 try   
		 * WSZExtracter extracter = new WSZExtracter(Constants.DEFAULT_SKIN_PATH);
		 *      extracter  
		 *       
		 */
		try{
			try{
				try{
					InputStream input = Main.class.getResourceAsStream("defaultSkin.wsz");
					File tmpFile = File.createTempFile("defaultSkin", ".skc");
					tmpFile.deleteOnExit();
					
					createTempFileFromStream(input, tmpFile);
					
					final SkinFilesManager extractor = 
						new SkinFilesManager(tmpFile);
					
					final IProject project = initializeProject(extractor);
					
					final SkinWriter projectWriter = 
						new SkinWriter(project.getSkin(), extractor);
					
					JMenuBar menubar = new JMenuBar();
					JMenu file = new JMenu("File");
					JMenu about = new JMenu("About");
					menubar.add(file);
					menubar.add(about);
			
			
					frame.setJMenuBar(menubar);
					final Viewer viewer = new Viewer(project);
					frame.getContentPane().add(viewer);
					frame.setJMenuBar(menubar);
					
					JMenuItem itemNew = new JMenuItem("New");
					
					itemNew.addActionListener(new ActionListener (){
					public void actionPerformed(ActionEvent arg0) {
						if (Viewer.isChanged()){
							manageSave(frame, extractor, 
									projectWriter, viewer, 2);
						}else{
							setNew(frame, extractor, projectWriter, viewer);
						}
					}
					});
		
					JMenuItem itemOpen = new JMenuItem("Open...");
					itemOpen.addActionListener(new ActionListener (){
					public void actionPerformed(ActionEvent arg0) {
						if (Viewer.isChanged()){
							manageSave(frame, extractor, 
									projectWriter, viewer, 1);
						} else {
							open(frame, extractor, projectWriter, viewer);
						}
					}
					});		
		
					JMenuItem itemSave = new JMenuItem("Save");
					itemSave.addActionListener(new ActionListener (){
					public void actionPerformed(ActionEvent arg0) {
						save(extractor, projectWriter, frame);
					}
					});
				
					JMenuItem itemSaveAs = new JMenuItem("Save As...");
					itemSaveAs.addActionListener(new ActionListener (){
					public void actionPerformed(ActionEvent arg0) {
						saveAs(extractor, projectWriter, frame);
					}
					});
				
					JMenuItem itemExit = new JMenuItem("Exit");
					itemExit.addActionListener(new ActionListener (){
					public void actionPerformed(ActionEvent arg0) {
						if (Viewer.isChanged()){
							manageSave(frame, extractor, 
									projectWriter, viewer, 3);
						} else {
							System.exit(0);
						}
					}
					});
					
					JMenuItem itemAbout = new JMenuItem("About");
					itemAbout.addActionListener(new ActionListener (){
					public void actionPerformed(ActionEvent arg0) {
						showAbout(frame);
					}
					});
					
					file.add(itemNew);
					file.add(itemOpen);
					file.add(itemSave);
					file.add(itemSaveAs);
					file.add(itemExit);
					about.add(itemAbout);
					frame.setVisible(true);
				}catch(CriticalError e){
					e.printStackTrace();
					showErrorDialog(frame, e.getMessage(), true);
				}
			}catch(ErrorReadingProject e){
				showErrorDialog(frame, "  /n--  WSZExtracter   ", true);
			}
		}catch(Exception e){
			showErrorDialog(frame, e.getMessage(), false);
		}
	}
}
