package org.amse.asves.skinCreator.wsz;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.io.*;

import org.amse.asves.skinCreator.model.ContainerName;
import org.amse.asves.skinCreator.exceptions.*;

/**
 *       
 * @author 
 *
 */
public class SkinFilesManager {
	
	private ZipFile mySkin, myDefaultSkin;
	private String myCurrentSkinPath;//, myDefaultSkinPath;
	private List<ZipEntry> mySkinEntries = new ArrayList<ZipEntry>();
	private List<ZipEntry> myDefaultSkinEntries = new ArrayList<ZipEntry>();
	private boolean myIsDefault = true;
	
	
	public boolean currentIsDefault(){
		return myIsDefault;
	}
	
	private List<ZipEntry> copyEntries (ZipFile zip){
		Enumeration en = zip.entries();
		ArrayList<ZipEntry> entries = new ArrayList<ZipEntry>();
		
		while(en.hasMoreElements())
		{
			entries.add(   (ZipEntry)en.nextElement()   );
		}
		return Collections.unmodifiableList(entries);
	}
	
	/**
	 *   
	 *     
	 */
	public void renew(){
		//myCurrentSkinPath = myDefaultSkinPath;
		myCurrentSkinPath = "";
		mySkinEntries = myDefaultSkinEntries;
		mySkin = myDefaultSkin;
		myIsDefault = true;
	}
	
	/**
	 *     ,        
	 * @throws ErrorReadingProject     
	 */
	public void refreshCurrentSkin() throws ErrorReadingProject{
		if (!currentIsDefault()){
			try {
				mySkin = new ZipFile (myCurrentSkinPath);
				//System.out.println(findValidEntry(mySkin, ContainerName.VOLUME));
			}catch(IOException e){
				throw new ErrorReadingProject("   ");
			}
			mySkinEntries = copyEntries(mySkin);
		}
	}
	
	/**
	 * 
	 *      ,    
	 *     @param currentSkinPath
	 * @throws ErrorReadingProject    
	 */
	public SkinFilesManager (File defaultSkin)
		throws ErrorReadingProject{
		
		try {
			myDefaultSkin = new ZipFile (defaultSkin);
		}catch(IOException e){
			e.printStackTrace();
			throw new CriticalError(e.getMessage());//+"   ");
		}
		
		mySkin = myDefaultSkin;
		mySkinEntries = copyEntries(mySkin);
		myDefaultSkinEntries = copyEntries(myDefaultSkin);
		
	}
	
	/**
	 *      @param currentSkinFile
	 * @throws ErrorReadingProject   
	 */
	public void setCurrentSkin(File currentSkinFile) throws ErrorReadingProject{
		myIsDefault = false;
		
		try {
			mySkin = new ZipFile (currentSkinFile);
		}catch(IOException e){
			e.printStackTrace();
			throw new ErrorReadingProject("   ");
		}
		mySkinEntries = copyEntries(mySkin);
		myCurrentSkinPath = currentSkinFile.getPath();
	}
	
	/**
	 * @return    @param zip
	 *          
	 *   ,        
	 * @throws CodeError -  
	 */
	public List<ZipEntry> getEntries (ZipFile zip) throws CodeError{
		if (zip.equals(myDefaultSkin)){
			return myDefaultSkinEntries;
		}else{
			if (zip.equals(mySkin)){
				return mySkinEntries;
			}else{
				throw new CodeError("");
			}
		}
	}
	
	/**
	 *     ,    
	 *     ?
	 * @return  @param path
	 */
	private String changeSlashesToSeparators(String path){
		  char[] chDst = new char[path.length()];
		  String tmp;
		    
		  for(int i = 0; i < path.length(); i++)
		  {
		    if((path.charAt(i) == '/') || (path.charAt(i) == '\\'))
		      chDst[i] = File.separatorChar;
		    else
		      chDst[i] = path.charAt(i);
		  }
		  tmp = new String(chDst);
		  return tmp;

	}
	
	/**
	 *      
	 * @return  @param path
	 */
	private String cutFolders(String path){
	    int index = path.lastIndexOf(File.separatorChar);
		int length = path.length();
		
		if (length <= 0) return "";
		//System.out.println(path);
	    if(index != -1){
			return path.substring(index + 1, length );
		} else{     
			return path;
		}
	}
	
	/**
	 *       
	 *  
	 * @param filePath
	 * @return
	 */
	private String getFileShortName(String filePath){
		return cutFolders(changeSlashesToSeparators(filePath));
	}
	
	/**
	 *   .     
	 *        @param entry   
	 * @return   
	 */
	private String getEntryShortName(ZipEntry entry) {
		return getFileShortName(entry.getName());
	}
	
	/**
	 *    @param name      @param zip
	 *             
	 *          
	 *  
	 * @return    null
	 * @throws CodeError    getEntries
	 */
	public ZipEntry findValidEntry(ZipFile zip, String name) throws CodeError{
		List<ZipEntry> entries = getEntries(zip);
		
		for (ZipEntry entry : entries){
			if (getEntryShortName(entry).toUpperCase().equals(name.toUpperCase())){
				return entry;
			}
		}
		return null;
	}
	
	/**
	 *   
	 *   @param zip       @param name, 
	 *     -
	 *        
	 * @see org.amse.asves.skinCreator.model.ContainerName
	 * @return    null
	 * @throws CodeError
	 */
	public ZipEntry findValidEntry(ZipFile zip, ContainerName name) throws CodeError{
		List<ZipEntry> entries = getEntries(zip);
		
		ZipEntry realNameEntry = findValidEntry(zip, name.getFileNameU());
		if (realNameEntry != null){
			return realNameEntry;
		}
		/**
		 *        
		 *   (emergency) 
		 */
		for (ZipEntry entry : entries){
			if (getEntryShortName(entry).toUpperCase().equals(name.getEmergencyContainer().getFileNameU())){
				return entry;
			}
		}
		return null;
		
	}
	
	/**
	 *         @param name
	 * @return  
	 * @throws CodeError      
	 * @throws ErrorReadingProject      
	 */
	public ZipFile getValidSkinForFileName(String name) throws CodeError, ErrorReadingProject{
		
		if (findValidEntry(mySkin, name) != null){
			return mySkin;
		}else{
			if (findValidEntry(myDefaultSkin, name) != null){
				return myDefaultSkin;
			}else{
				throw new ErrorReadingProject("    "
						+ name + "       !");
			}
				
		}
		
	}
	
	/**
	 *     
	 * @param name
	 * @return
	 * @throws CodeError
	 * @throws ErrorReadingProject
	 */
	public ZipFile getValidSkinForContainerName(ContainerName name) throws CodeError, ErrorReadingProject{
		
		if ((findValidEntry(mySkin, name) != null)){
			return mySkin;
		}else{
			if ((findValidEntry(myDefaultSkin, name) != null)){
				return myDefaultSkin;
			}else{
				throw new ErrorReadingProject("    "
						+ name + "       !");
			}
		}
		
	}
	
	/**
	 *             
	 * 
	 */
	public ZipFile getDefaultSkin(){
		return myDefaultSkin;
	}
	
	public String getCurrentSkinFileName(){
		return myCurrentSkinPath;
	}
	
	public ZipFile getCurrentSkin(){
		return mySkin;
	}
	
	/**
	 *   @param entry   @param zip
	 *           
	 *   @param where
	 * @throws ExtractError    
	 */
	/*
	 * TODO         
	 *          
	 */
	public File extractEntry(ZipFile zip, ZipEntry entry) throws ErrorReadingProject{
		if (!entry.isDirectory()){
			
       	 	File file;
       	 	try{
       	 		file = File.createTempFile(getEntryShortName(entry), ".skc");
       	 		file.deleteOnExit();
       	 	}catch(IOException e){
       	 		throw new ExtractError("   ");
       	 	}
       	 	
       	 	FileOutputStream fos;
        
       	 	try{
        		fos = new FileOutputStream(file);
       	 	}catch(IOException e){
       	 		throw new ExtractError("       " + file.getName());
       	 	}
        
       	 	InputStream is = null;
        
       	 	try{
       	 		is = zip.getInputStream(entry);
       	 	}catch(IOException e){
       	 		throw new ExtractError("       zip " + entry.getName());
       	 	}
		
       	 	if (is != null){
       	 		try{
       	 			copyStream (is,fos);
       	 		}catch(CannotCopyStreamException e){
       	 			throw new ExtractError(e.getMessage());
       	 		}
	    	
       	 		try{
       	 			is.close();
       	 		}catch(IOException e){
       	 			throw new ExtractError("       zip " + entry.getName());
       	 		} 
       	 	}
       	 	try{
       	 		fos.close();
       	 	}catch(IOException e){
       	 		throw new ExtractError("       " + file.getName());
       	 	}
       	 	return file;
	    }
		return null;
	}
	
	public static void copyStream(InputStream in, OutputStream out)
		throws CannotCopyStreamException{
		
		byte[] buf = new byte[1024];

	 	int nLength;
	 	try{
	 		while(true) {
	 			nLength = in.read(buf);
	 			if(nLength < 0)  
			break;
	 			out.write(buf, 0, nLength);
	 		}
	 	}catch(IOException e){
	      
	 		throw new CannotCopyStreamException("    ");
	 	}
	}
	
	public static String getTempSourceFileName(String name){
		int x = name.indexOf(".");
		return name.substring(0,x + 4);
	}
}
